import json
import csv

def write_to_csv(fields, content, csvfile="output.csv"):
  """Write content into csv file

  Args:
    fields (list of str): heads of content
    content (list of dict): one item in content is one line in csv file
    csvfile (str): csv file path
  """
  with open(csvfile, 'w') as output_file:
    dict_writer = csv.DictWriter(output_file, restval="-", fieldnames=fields, delimiter='\t')
    dict_writer.writeheader()
    dict_writer.writerows(content)

def write_to_json(content, jsonfile="output.json"):
  """Write content into json file

  Args:
    content (list of dict): content to be written into json file
    jsonfile (str): json file path
  """
  with open(jsonfile, 'w') as outfile:
    json.dump(content, outfile)


class CSVWriter:
  """Write object into csv file.

  Attributes:
    dstfile (str): destnation file name with extension
  """
  def __init__(self, name):
    """Initalize ``dstfile``

    Args:
      name (str): file name WITHOUT extension
    """
    self.dstfile = name + '.csv'

  def _encode(self, content):
    """Encode a list of dict, serialization.

    Args:
      content (list of dict): original content

    Returns:
      list of dict: serialized content
    """
    new_content = []
    for e in content:
      new_e = {}
      for k,v in e.items():
        new_e[k] = json.dumps(v) if not isinstance(v, str) and not isinstance(v, int) and v != None else v
      new_content.append(new_e)
    return new_content

  def write(self, content):
    """Write content into csv

    Args:
      content (list of dict): content to be written into csv file
    """
    _content = self._encode(content)
    keys = _content[0].keys() if len(_content) > 0 else []
    write_to_csv(keys, _content, self.dstfile)




from sencore import VocabParser
from x2cdict import VocabDict

class VocabAnalyzer:
  """Analyze vocabulary of sentences 
  """
  def __init__(self, lang):
    """Initialize ``parser`` and ``dictAPI`` by language

    Args:
      lang (str): language abbreviation, e.g, ``en``, ``es``
    """

    self._parser = VocabParser(lang)
    self._dictAPI = VocabDict(lang, "cn")

  def _vocabs(self, sens):
    """Get vocabularies from sentences, may with duplicates.
    
    Args:
      sens (list): sentences

    Returns:
      list: vocabularies
    """
    vs = []
    for s in sens:
      vs = vs + self._parser.digest(s)
    return vs

  def _unique_vocabs(self, sens):
    """Get vocabularies from sentences, WITHOUT duplicates.

    Args:
      sens (list): sentences

    Returns:
      list: vocabularies
    """
    result = []
    vs = self._vocabs(sens)
    for e in vs:
      if e not in result:
        result.append(e)
    return result

  def overview_vocabs(self, sens, google=False):
    """Get vocabularies with explanation from sentences, WITHOUT duplicates.

    Args:
      sens (list): sentences

    Returns:
      list: vocabularies
    """
    result = []
    vs = self._unique_vocabs(sens)
    for v in vs:
      r = self._dictAPI.search(v["word"], v["pos"], google)
      if r != None:
        result.append(r)
    return result





"""
.. topic:: Analyze vocabularies

  .. code:: python

    from plaintext_analyzer import PlaintextReader
    from plaintext_analyzer import VocabAnalyzer

    sf = PlaintextReader(source, stype, lang)
    sens = sf.sentences
    print(sens)

    analyzer = VocabAnalyzer(lang)
    exs = analyzer.overview_vocabs(sens, google)
    print(exs)

.. topic:: Analyze phrases

  .. code:: python
 
    from plaintext_analyzer import PlaintextReader
    from plaintext_analyzer import PhraseAnalyzer

    sf = PlaintextReader(source, stype, lang)
    sens = sf.sentences
    print(sens)

    analyzer = PhraseAnalyzer(lang)
    exs = analyzer.overview_phrases(sens, google)
    print(exs)
"""

__version__ = '0.1.2'

from .file_reader import PlaintextReader
from .file_writer import CSVWriter
from .vocab_analyzer import VocabAnalyzer
from .phrase_analyzer import PhraseAnalyzer



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['plaintext_analyzer']

package_data = \
{'': ['*']}

install_requires = \
['click>=7.1.2,<8.0.0', 'sencore>=0.1.7,<0.2.0', 'x2cdict>=0.1.40,<0.2.0']

entry_points = \
{'console_scripts': ['pta_phrase = plaintext_analyzer.entry:parser_phrase',
                     'pta_vocab = plaintext_analyzer.entry:parser_vocab']}

setup_kwargs = {
    'name': 'plaintext-analyzer',
    'version': '0.1.2',
    'description': '',
    'long_description': '# Installation from pip3\n\n```shell\npip3 install --verbose plaintext_analyzer \npython -m spacy download en_core_web_trf\npython -m spacy download es_dep_news_trf\n```\n\n# Usage\n\nPlease refer to [api docs](https://qishe-nlp.github.io/plaintext-analyzer/).\n\n### Excutable usage\n\n* Get vocabularies from plaintext file \n\n```shell\npta_vocab --source en_plaintext.txt --stype FILE --lang en  \n``` \n\n* Get vocabularies from text \n\n```shell\npta_vocab --source "The typical Bangladeshi breakfast consists of flour-based flatbreads such as chapati, roti or paratha, served with a curry. Usually the curry can be vegetable, home-fried potatoes, or scrambled eggs. The breakfast varies according to location and the eater\'s income. In villages and rural areas, rice with curry (potato mash, dal ) is mostly preferred by day laborers. In the city, sliced bread with jam or jelly is chosen due to time efficiency. In Bangladesh tea is preferred to coffee and is an essential part of most breakfasts. Having toasted biscuits, bread or puffed rice with tea is also very popular." --stype RAW --lang en  \n``` \n\n* Get vocabularies from plaintext file, and write to csv files \n\n```shell\npta_vocab --source en_plaintext.txt --stype FILE --lang en --dstname en_vocab\n``` \n\n* Get vocabularies from text, and write to csv file \n\n```shell\npta_vocab --source "The typical Bangladeshi breakfast consists of flour-based flatbreads such as chapati, roti or paratha, served with a curry. Usually the curry can be vegetable, home-fried potatoes, or scrambled eggs. The breakfast varies according to location and the eater\'s income. In villages and rural areas, rice with curry (potato mash, dal ) is mostly preferred by day laborers. In the city, sliced bread with jam or jelly is chosen due to time efficiency. In Bangladesh tea is preferred to coffee and is an essential part of most breakfasts. Having toasted biscuits, bread or puffed rice with tea is also very popular." --stype RAW --lang en --dstname en_vocab \n``` \n\n* Get phrases from plaintext file \n\n```shell\npta_phrase --source en_plaintext.txt --stype FILE --lang en  \n``` \n\n* Get phrases from text \n\n```shell\npta_phrase --source "The typical Bangladeshi breakfast consists of flour-based flatbreads such as chapati, roti or paratha, served with a curry. Usually the curry can be vegetable, home-fried potatoes, or scrambled eggs. The breakfast varies according to location and the eater\'s income. In villages and rural areas, rice with curry (potato mash, dal ) is mostly preferred by day laborers. In the city, sliced bread with jam or jelly is chosen due to time efficiency. In Bangladesh tea is preferred to coffee and is an essential part of most breakfasts. Having toasted biscuits, bread or puffed rice with tea is also very popular." --stype RAW --lang en  \n``` \n\n* Get phrases from plaintext file, and write to csv files \n\n```shell\npta_phrase --source en_plaintext.txt --stype FILE --lang en --dstname en_phrase\n``` \n\n* Get phrases from text, and write to csv file \n\n```shell\npta_phrase --source "The typical Bangladeshi breakfast consists of flour-based flatbreads such as chapati, roti or paratha, served with a curry. Usually the curry can be vegetable, home-fried potatoes, or scrambled eggs. The breakfast varies according to location and the eater\'s income. In villages and rural areas, rice with curry (potato mash, dal ) is mostly preferred by day laborers. In the city, sliced bread with jam or jelly is chosen due to time efficiency. In Bangladesh tea is preferred to coffee and is an essential part of most breakfasts. Having toasted biscuits, bread or puffed rice with tea is also very popular." --stype RAW --lang en --dstname en_phrase \n``` \n\n### Package usage\n```\ndef parser_vocab(source, stype, lang):\n\n  sf = PlaintextReader(source, stype, lang)\n  sens = sf.sentences\n\n  analyzer = VocabAnalyzer(lang)\n  exs = analyzer.overview_vocabs(sens)\n\n  print(exs)\n\ndef parser_phrase(source, stype, lang):\n\n  sf = PlaintextReader(source, stype, lang)\n  sens = sf.sentences\n\n  analyzer = PhraseAnalyzer(lang)\n  exs = analyzer.overview_phrases(sens)\n\n  print(exs)\n\n```\n\n# Development\n\n### Clone project\n```\ngit clone https://github.com/qishe-nlp/plaintext-analyzer.git\n```\n\n### Install [poetry](https://python-poetry.org/docs/)\n\n### Install dependencies\n```\npoetry update\n```\n\n### Test\n```\npoetry run pytest -rP\n```\nwhich run tests under `tests/*`\n\n### Execute\n```\npoetry run pta_vocab --help\npoetry run pta_phrase --help\n```\n\n### Create sphinx docs\n```\npoetry shell\ncd apidocs\nsphinx-apidoc -f -o source ../subtitlecore\nmake html\npython -m http.server -d build/html\n```\n\n### Hose docs on github pages\n```\ncp -rf apidocs/build/html/* docs/\n```\n\n### Build\n* Change `version` in `pyproject.toml` and `plaintext_analyzer/__init__.py`\n* Build python package by `poetry build`\n\n### Git commit and push\n\n### Publish from local dev env\n* Set pypi test environment variables in poetry, refer to [poetry doc](https://python-poetry.org/docs/repositories/)\n* Publish to pypi test by `poetry publish -r test`\n\n### Publish through CI \n\n* Github action build and publish package to [test pypi repo](https://test.pypi.org/)\n\n```\ngit tag [x.x.x]\ngit push origin master\n```\n\n* Manually publish to [pypi repo](https://pypi.org/) through [github action](https://github.com/qishe-nlp/plaintext-analyzer/actions/workflows/pypi.yml)\n\n',
    'author': 'Your Name',
    'author_email': 'you@example.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://github.com/qishe-nlp/plaintext-analyzer',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<4.0',
}


setup(**setup_kwargs)



from plaintext_analyzer import PlaintextReader
from plaintext_analyzer import VocabAnalyzer, PhraseAnalyzer
from plaintext_analyzer import CSVWriter

import click
import json

#pta_file2phrase = "plaintext_analyzer.entry:file2phrase"

#pta_text2vocab = "plaintext_analyzer.entry:text2vocab"
#pta_text2phrase = "plaintext_analyzer.entry:text2phrase"


@click.command()
@click.option("--source", help="Specify the filename or plaintext itself", prompt="source")
@click.option("--stype", help="RAW or FILE", prompt="source type[TEXT|FILE]")
@click.option("--lang", help="Specify the language", default="en", prompt="language")
@click.option("--dstname", required=False, help="Specify the output csv name", default=None)
@click.option("--google", required=False, help="Whether extra help needed", type=bool, default=False)
def parser_vocab(source, stype, lang, dstname, google):

  phase = {"step": 1, "msg": "Start sentenizing"}
  print(json.dumps(phase), flush=True)

  sf = PlaintextReader(source, stype, lang)
  sens = sf.sentences

  phase = {"step": 2, "msg": "Finish sentenizing"}
  print(json.dumps(phase), flush=True)

  analyzer = VocabAnalyzer(lang)
  exs = analyzer.overview_vocabs(sens, google)
  simple_exs = [{k:v for k,v in e.items() if k in ['word', 'dict_pos', 'meaning']} for e in exs]
  shown = simple_exs[:20]

  phase = {"step": 3, "msg": "Finish vocabs dictionary lookup", "vocabs": shown}
  print(json.dumps(phase), flush=True)

  if dstname:
    csv_writer = CSVWriter(dstname)
    csv_writer.write(simple_exs)
    csv_writer = CSVWriter(dstname+'.forpptx')
    csv_writer.write(exs)

    phase = {"step": 4, "msg": "Finish csv saving"} 
    print(json.dumps(phase), flush=True)


@click.command()
@click.option("--source", help="Specify the filename or plaintext itself", prompt="source")
@click.option("--stype", help="RAW or FILE", prompt="source type[TEXT|FILE]")
@click.option("--lang", help="Specify the language", default="en", prompt="language")
@click.option("--dstname", required=False, help="Specify the output csv name", default=None)
def parser_phrase(source, stype, lang, dstname):

  phase = {"step": 1, "msg": "Start sentenizing"}
  print(json.dumps(phase), flush=True)

  sf = PlaintextReader(source, stype, lang)
  sens = sf.sentences

  phase = {"step": 2, "msg": "Finish sentenizing"}
  print(json.dumps(phase), flush=True)

  analyzer = PhraseAnalyzer(lang)
  exs = analyzer.overview_phrases(sens)

  phase = {"step": 3, "msg": "Finish vocabs dictionary lookup", "vocabs": exs[:3]}
  print(json.dumps(phase), flush=True)

  if dstname:
    csv_writer = CSVWriter(dstname+'.forpptx')
    csv_writer.write(exs)

    phase = {"step": 4, "msg": "Finish csv saving"} 
    print(json.dumps(phase), flush=True)


